/*
Copyright 2006, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2006
*/

/** \file ScatterWeb.Net.h
 ** \ingroup System
 ** \brief The ScatterWeb.Net implements a network service that enables you to
 ** develop applications that use network resources without worrying about the
 ** specific details of the individual protocol.
 ** <p>
 ** General
 ** Radio software supports acknowledgements, so it is important to know that
 ** all packet types which are no broadcast will get acknowledged by radio
 ** software. The radio software uses
 ** the num field for acknowledged packets to recognize doubles. 
 ** <p>
 ** packet_t is a struct containing a packet header fields and pointers to
 ** packet data and is used for sending & receiving. 
 ** <p>
 ** As a packet may be often constructed out of a header portion and a data
 ** portion, packet_t supports two data pointers to get passed in for sending,
 ** so these portions don't need to be copied together. When receiving however,
 ** data pointer contains header and data consecutively and header and
 ** header_length are null.
 ** <p>
 ** How to receive packets
 ** To receive packets of a specific type one must define a handler function and
 ** add it to Net_txHandler for firmware handlers or to the C_RADIO app callback
 ** function. 
 ** <p>
 ** When the handler is then called, the function can access the received packet
 ** in rxPacket. 
 ** <p>
 ** How to send packets
 ** For sending packets a packet_t must be initialized:
 ** - to must contain the destination id of receiver or broadcast address.
 ** - type must contain the packet type.
 ** - header/data must be set to NULL or to the data that should get
 **   transported. If set, header_length/data_length should contain the length
 **   of the data.
 ** - num can be set to transport a byte for not acknowledged packets, otherwise
 **   it will get overwritten.
 ** - from is always overwritten with the id of this device.
 ** <p>
 ** Such a packet can then be queued into tx ring buffer with Net_send and a
 ** pointer to initialized packet_t.
 ** <p>
 ** Additionally these function accept a callback pointer to functions
 ** which accept an UINT8. If set, this callback is called when the packet is
 ** removed from the tx ring buffer:
 ** - For unacknowledged packets this is after sending the packet.
 ** - For acknlowedged packets this is after successfully receiving an ACK for
 **   this packet (callback argument is set to 1) or when all retries are made
 ** without sucess (callback argument is set to 0). 
 ** 
 ** @author        Marco Hutta
 ** @version       1.0
 ** @since         1.0     
 **/

#ifndef ScatterWeb_Net
#define ScatterWeb_Net

/**
 * Predefined Address: Broadcast Address.
 *
 * @since         1.0
 */
#define BROADCAST       (0xFFFF)

/**
 * Predefined Address: Illegal Address.
 *
 * @since         1.0
 */
#define UNKNOWN         (0x0000)

/**
 * Packet Type: Acknowledgment packet.
 *
 * @since         1.0
 */
#define ACK_PACKET (0x00)

/**
 * Packet Type: Ping packet.
 *
 * @since         1.0
 */
#define PING_PACKET (0x02)

/**
 * Packet Type: Pong packet.
 *
 * @since         1.0
 */
#define PONG_PACKET (0x03)

/**
 * Packet Type: Module Messaging.
 *
 * @since         1.0
 */
#define MESSAGING_REQUEST_PACKET (0x04)

/**
 * Packet Type: Module Messaging - Answer MESSAGING_REQUEST_PACKET.
 *
 * @since         1.0
 */
#define MESSAGING_REPLY_PACKET (0x05)

/**
 * Struct for the packet
 * <ul>
 * <li>to - Destination.
 * <li>from - Source (automatically set by radioSend).
 * <li>type - Packet type.
 * <li>num - Packet number.
 * <li>header - Pointer to packet extra header.
 * <li>header_length - 0 means no extra header (in radioReceive always 0)
 * <li>data - Pointer to packet data.
 * <li>data_length - Length of data.
 * </ul>
 *  
 * @since         1.0
 */
typedef struct {
  UINT16 to;
  UINT16 from;
  UINT8 type;
  UINT8 num;
  UINT8* header;
  UINT8 header_length;
  UINT8* data;
  UINT8 data_length;
} packet_t;

/**
 * Struct for received packet
 *
 * @since         1.0
 */
extern volatile packet_t rxPacket;

/**
 * Struct for sended packet
 *
 * @since         1.0
 */
extern volatile packet_t sentPacket; 

/**
 * Number of retries done (retries needed to deliver a packet).
 *
 * @since         1.0
 */
extern volatile UINT16 timesRepeated;

/**
 * Define that enables CSMA/random backoff.
 *
 * @see           #Net_on(UINT8 mode)
 * @since         1.0
 */
#define TXMODE_BACKOFF  (0x01)

/**
 * Define that turns the Radio off when nothing more is to sent.
 *
 * @see           #Net_on(UINT8 mode)
 * @since         1.0
 */
#define TXMODE_LP       (0x02)

/**
 * Initializes the radio
 *
 * @return          <code>True</code> always.
 * @since           1.0
 */
bool Net_init();

/**
 * Enables the radio
 *
 * @param mode      mode to set
 * @return          <code>True</code> always.
 * @since           1.0
 */
bool Net_on(UINT8 mode);

/**
 * Disables the radio
 *
 * @return          <code>True</code> always.
 * @since           1.0
 */
bool Net_off();

/**
 * Returns the RSSI value of the given node
 *
 * @param node      node from whom the RSSI value shall be
 * @return          RSSI value of the given node
 * @since           1.0
 */
UINT8 Net_getRSSI(UINT16 node);

/**
 * Sets the transceiver power
 *
 * @param value     tx power to set in hex
 * @return          <code>True</code> always.
 * @since           1.0
 */
bool Net_setTxPower(UINT8 value);

/**
 * Sends a packet over radio
 *
 * @param packet    packet to send
 * @param callback  will call after sending (without: use NULL)
 * @return          <code>True</code> if packet is successfully buffered;
 *                  <code>False</code> otherwise. 
 * @since           1.0
 */
bool Net_send(packet_t* packet, fp_char_t callback);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
bool Net_changeMode(UINT8 mode);

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
bool Net_rxHandler();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
bool Net_txHandler();

#endif
