/*
Copyright 2006, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2006
*/

/** \mainpage
 *
 * <p>
 * <h1>Welcome to ScatterWeb</h1>
 */

/** \file ScatterWeb.Configuration.h
 ** \ingroup System
 ** \brief The ScatterWeb.System provides fuctions of the core system.
 ** <p>
 ** <h1>Hardware & Software watchdog, reset reason, stack check and panics.</h1>
 ** <p>
 ** This module contains two macros to control the hardware watchdog.
 ** Furthermore it supports determining the reset source, a simple
 ** stack checker to avoid stack overflows and a panic function.
 ** <p>
 ** <h1>Hardware&Software Watchdog</h1>
 ** <p>
 ** The function System_startWatchdog() allows starting or resetting both watchdogs, System_stopWatchdog() stops it.
 ** These macros are mostly for firmware internal use.
 ** In the perfect case noone should call System_stopWatchdog() and the only place to
 ** call System_startWatchdog() repeatedly would be the main loop.
 ** However some software parts need a long time or must run exclusively (then also
 ** disabling the interrupts together with System_stopWatchdog() )
 ** <p>
 ** Timer_A1 Interrupt was used to implement a software watchdog with a timer interrupt
 ** running together with the built in hardware watchdog, because an interrupt
 ** offers the possibility to receover the address where the watchdog timeout occured.
 ** This address gets written in a panic message, when Timer_A1 Interrupt fires (what means watchdog timeout).
 ** Putting the hardware watchdog into interrupt mode (what is possible with the MSP430)
 ** is not a good idea, because then it can't recover the MCU in a case, where it
 ** is not running at all (for example MSP430s known startup problems with a slow or noisy voltage rise
 ** which lead to such a situation).
 ** <p>
 ** <h1>Reset source</h1>
 ** <p>
 **  The function _System_initWD() runs at the very beginning of main just after initPorst().
 ** It checks some registers to see where the reset came from and then sets wd_startup
 ** acoordingly.
 ** <p>
 ** Technical details of _System_initWD():
 ** <p>
 ** The MSP internally has two reset signals (slau049.pdf, Chapter 3):
 ** - POR (power-on reset): This is a device reset. Set by Power-Up or a low RST line
 **   when configured in reset mode (standard).
 ** - PUC (power-up clear); A PUC follows every POR. But it can occur also without
 **   because of watdchdog timer expiry or flash control register key violation.
 ** <p>
 ** The SFRs are initialized on resets and some bits get only set or reset,
 ** when a POR occured, not on a PUC only (these in braces, see slau049.pdf, page 3-4).
 ** <p>
 ** This feature is used by wdt_init with the register TACTL to detect a
 ** POR since it will only get reset (complete zero) if a POR occured.
 ** <p>
 ** If a PUC only occured it was a warm start. The function looks further
 ** wether it was the WDT, Flash key violation or an unknown source.
 ** If it was a Flash key violation it is checked with var wd_state
 ** wether it was wanted by software (System_reset() with wd_state==WDS_SWRESET
 ** or panic() with wd_state==WDS_PANIC) or a real key violation.
 ** <p>
 ** <h1>Stack checking</h1>
 ** <p>
 ** _System_checkStack is a small function which once called by main runs
 ** once every second with a timer. It checks 8 bytes on top
 ** of the data segment (behind __noinit_end provided by the linker script)
 ** for corruption. The reset_vector in main() fills the memory
 ** from __noinit_end - 0x09FF with 0xABCD, so the procedure simply
 ** checks the four ints from __noinit_end for the value 0xABCD. If they
 ** are not there, the stack must have gone that low and possibly lower.
 ** The procedure will panic then.
 ** <p>
 ** <h1>Panic</h1>
 ** <p>
 ** System_panic is a function to call when some bad condition was detected
 ** (for example low stack, full timers).
 ** System_panic writes a small info msg into second info memory segment (0x1080-0x10FF)
 ** and then resets the controller.
 ** Multiple panic messages can be written until the segment is full.
 ** <p>
 ** One could go further and implement a complete memory dump into flash, what
 ** would give the possibility to examine the complete system state later for debugging.
 ** <p>
 ** <h1>DA-Converter</h1>
 ** <p>
 ** DAC, RF_POWER is on P67
 ** The DAC12 outputs are multiplexed with the port P6 pins and ADC12 analog
 ** inputs. When DAC12AMPx > 0, the DAC12 function is automatically selected
 ** for the pin, regardless of the state of the associated P6SELx and P6DIRx bits.
 ** <p>
 ** DAC12AMP      typical/max current      calibration time
 **    0                OFF                      OFF
 **    1                OFF                      OFF
 **    2               50/110                    100
 **    3                 ?                        32
 **    4                 ?                         6
 **    5              200/440                     32
 **    6                 ?                         6
 **    7              700/1500                     6
 ** <p>
 ** Setup DAC:
 ** SREF = 0  : VRef+
 ** RES  = 0  : 12Bit
 ** LSEL = 0  : latch loads when dat is written
 ** IR   = 1  : full-scale = 1*refvoltage
 ** AMP  = 7  : high speed / current
 ** DF  = 0   : straight binary
 ** IE  = 0   : interrupt disabled
 ** IFG  = 0  : *dont care*
 ** ENC  = 0  : disabled, not needed
 ** GRP  = 0  : not grouped
 ** <p>
 ** IMPORTANT: For VRef+ REF_ON and REF2_5V must be enabled in ADC Unit!!!
 ** VeREF+ is not connected?!
 ** <p>
 ** ADC12CTL0 |= REFON + REF2_5V; \
 ** <p>
 ** DAC12SREF_0 und DAC12SREF_1: Vref+
 ** DAC12SREF_2 und DAC12SREF_3: VEref+
 ** <p>
 ** <h1>Reset & Watchdog control.</h1>
 ** <p>
 ** High byte of WDTCTL must be written as 0x5A (WDTPW) to avoid a reset.
 ** High byte of WDTCTL always reads as 0x69, so WDTCTL |= wont work and
 ** generate resets!
 ** <p>
 ** use System_startWatchdog() to start the WDT and to keep the watchdog alive at least
 ** 1000 ms after the last System_startWatchdog() call; or stop the Watchdog, else => reset
 ** 1000ms, WDT is clocked by fACLK (assumed 32KHz)
 ** <p>
 ** <h1>Subroutine to set DCO to selected frequency</h1>
 ** <p>
 ** The DCO is set via BCSCTL[2..0] (nominal range) and DCOCTL[7..0] (fine tuning)
 ** so minimum DCO is BCSCTL = 0, DCTOCTL = 0 and maximum is BCSCTL = 3, DCOCTL = FF
 ** BCSCTL[7] = XT2Off must also be 1 to disable XT2 (not available on our boards)
 ** the nominal ranges are overlapping, so 2,45768 Mhz reachable via
 ** BCSCTL = 6, DCOCTL = A8 or BCSCTL = 7, DCOCTL = 16. Its accuracy
 ** depends on temperature, voltage, device...
 ** <p>
 ** Initalizing & Controlling the DCO (Digitally Controlled Oscillator)
 ** <p>
 ** If ::FF_DCOCHECKER in config_t.firmwareFlags is set Timers_init will 
 ** add a timer every 100 ms for checkDCO and enable Timer_B1 interrupt. 
 ** <p>
 ** The interrupt continually measures the ACLK with the DCO (like in System_setDCO)
 ** and writes the result into dcoDiff.
 ** checkDCO takes samples every 100 ms of dcoDiff and adjusts
 ** the DCO if needed.
 ** <p>
 ** As the DCO is temperature-sensitive and both serial and radio
 ** communication depend on the DCO calibrating the DCO continously
 ** is recommended. Serial communication stops working at a temperature
 ** difference of ~10C.
 ** <p>
 ** A more low power option might be to not use checkDCO and just call
 ** System_setDCO in long intervals (a minute or even longer).     
 **
 ** @author        Marco Hutta
 ** @version       1.0
 ** @since         1.0
 **/  

#ifndef ScatterWeb_System
#define ScatterWeb_System

/**
 * Define WAKEUPLPM1
 *
 * @since         1.0
 */
#define WAKEUPLPM1(bit)  do {     \
    runModule |= bit; \
    CACTL1 |= CAIFG; \
    }while(0)

/**
 * Define READ_PC
 *
 * @since         1.0
 */
#define READ_PC \
({				\
	uint16_t __x;		\
	__asm__ __volatile__(	\
	"mov	r0, %0"		\
	: "=r" ((uint16_t)__x)	\
	:);			\
	__x;			\
})

/**
 * Define SET_ISR
 *
 * @since         1.0
 */
#define SET_ISR(isrvector, isrpointer)  do{ IVT[isrvector/2] = isrpointer; }while(0)

/**
 * Define GET_ISR
 *
 * @since         1.0
 */
#define GET_ISR(isrvector) ({ IVT[isrvector/2]; })


/**
 * Buffer size: Maximum size for normal terminal replies.
 *
 * @since         1.0
 */
#define TERM_BUFSIZE        (82)

/**
 * Buffer size: Maximum size of a line received by uart, rest will be cut.
 *
 * @since         1.0
 */
#define SERIAL_BUFSIZE      (82)

/**
 * Buffer size: Maximum size of packets that can be received, header included is
 * 8 bytes.
 *
 * @since         1.0
 */
#define RADIO_RXBUF_SIZE    (270)

/**
 * Buffer size: Size of the TX ring buffer.
 *
 * @since         1.0
 */
#define RADIO_TXBUF_SIZE    (400)

/**
 * Buffer size: Maximum number of timers, each timer requires 6 bytes of RAM
 * (==sizeof(timerEntry)).
 *
 * @since         1.0
 */
#define MAX_TIMERS          (20)

/**
 * Module flag for low power management: Set if Net_rxHandler should be called.
 *
 * @since         1.0
 */
#define MF_RADIO_RX   (0x0002)

/**
 * Module flag for low power management: Set if Net_txHandler should be called.
 *
 * @since         1.0
 */
#define MF_RADIO_TX   (0x0004)

/**
 * Module flag for low power management: Set if Timers_eventHandler should be
 * called.
 *
 * @since         1.0
 */
#define MF_TIMER      (0x0008)

/**
 * Module flag for low power management: Set if a serial line is complete.
 *
 * @since         1.0
 */
#define MF_SERIAL     (0x0010)

/**
 * Module flag for low power management
 *
 * @since         1.0
 */
#define MF_SENSOR     (0x0020)

/**
 * Define special section definition for linking
 *
 * @since         1.0
 */
#define INFOMEM            __attribute__((section(".infomem")))

/**
 * Callback type: Gets called everytime a whole line is received via serial port.
 *
 * @see           #System_registerCallback()
 * @since         1.0
 */
#define C_SERIAL              0

/**
 * Callback type: Gets called everytime an unknown packet (not handled by the
 * firmware) is received.
 *
 * @see           #System_registerCallback()
 * @since         1.0
 */
#define C_RADIO               1

/**
 * Callback type: Gets called for sensor values.
 *
 * @see           #System_registerCallback()
 * @since         1.0
 */
#define C_SENSOR              2

/**
 * The callbacks[] Array will get this size.
 *
 * @see           #callbacks[] 
 * @since         1.0
 */
#define NUMBER_CALLBACKS	  3

/**
 * Enum
 * <ul>
 * <li>True
 * <li>False
 * </ul> 
 *
 * @see           #bool 
 * @since         2.0
 */
enum {
  True = 1,
  False = 0,
};

/**
 * Bool
 * 
 * @since         2.0
 */
typedef char bool;

/**
 * Elementary data type
 * 
 * @since         1.0
 */
typedef unsigned char     UINT8;

/**
 * Elementary data type
 * 
 * @since         1.0
 */
typedef unsigned int      UINT16;

/**
 * Elementary data type
 * 
 * @since         1.0
 */
typedef unsigned long     UINT32;

/**
 * Elementary data type
 * 
 * @since         1.0
 */
typedef signed char       INT8;

/**
 * Elementary data type
 * 
 * @since         1.0
 */
typedef signed int        INT16;

/**
 * Elementary data type
 * 
 * @since         1.0
 */
typedef signed long       INT32;

/**
 * Several predefined function pointer type: For functions like: void
 * function(void).
 * 
 * @since         1.0
 */
typedef void(*fp_t)();

/**
 * Several predefined function pointer type: For functions like: void
 * function(UINT8)
 * 
 * @since         1.0
 */
typedef void(*fp_char_t)(UINT8);

/**
 * Several predefined function pointer type: For functions like: void
 * function(void*).
 * 
 * @since         1.0
 */
typedef void(*fp_vp)(void*);

/**
 * Describes data portions in the flash memory (firmware or application part) 
 * by start and end address, type, name and date. 
 * <ul>
 * <li>key
 * <li>start - Start address of reprogrammable image.
 * <li>end - End address of reprogrammable image.
 * <li>type
 * <li>name - Must be 20 chars max (with 0byte 21 alltogether).
 * <li>data - Filled with current date and time string by preprocessor.
 * </ul>
 *  
 * @since         1.0
 */
typedef struct {
  UINT16 key;
  void*  start;
  void*  end;
  UINT16 type;
  UINT8  name[21];
  UINT8  date[23];
} imageData_t;

/**
 * Temperature type (built on a signed int). It's a signed (twos complement) 
 * fixed point value with 8 bits before comma and 7 bits after. So Bit 15 is
 * sign, Bit14-7 is before comma and Bit 6-0 after comma. 
 * 
 * @since         1.0
 */
typedef union { unsigned int u; signed int s; } temp_t; 

/**
 * Pointer defined by the linker script at the application start.
 *
 * @since         1.0
 */
extern int __app_start;

/**
 * Pointer defined by the linker script at the application end.
 *
 * @since         1.0
 */
extern int __app_end;

/**
 * Pointer defined by the linker script at the firmware start.
 *
 * @since         1.0
 */
extern int __fw_start;

/**
 * Pointer defined by the linker script at the firmware end.
 *
 * @since         1.0
 */
extern int __fw_end;

/**
 * Software header. 
 * Theses variables are defined by the linker script, they are needed in the software header
 *
 * @since         1.0
 */
extern const imageData_t firmwareData;

/**
 * UINT16
 *
 * @since         1.0
 */
extern volatile UINT16 runModule;

/**
 * Table of interrupts
 *
 * @since         1.0
 */
extern void* volatile IVT[16];

/**
 * Table where callbacks are hold
 *
 * @since         1.0
 */
extern fp_vp callbacks[];

/**
 * Waits - A simple looping waiting function. Rough estimate at 2,4576 MHz: w=1
 * => 18us, w=10 => 62us, w=100 => 500us  w=400 => 4ms.
 * This method has two big disadvantages: it blocks and it's not accurate,
 * timers are better.
 *
 * @param w       ticks to wait 
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool System_wait(UINT32 w);

/**
 * Waits ms milliseconds with help of timer a (1024 ticks)
 *
 * @param ms      milliseconds to wait
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool System_delay(unsigned int ms);

/**
 * Sets DCO
 *
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool System_setDCO();

/**
 * Resets System
 *
 * @since         1.0
 */
void System_reset();         

/**
 * Registers callback
 *
 * @param type    type of callback
 * @param fp      function to call 
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool System_registerCallback(UINT8 type, void(*fp_t)()); 

/**
 * Starts watchdog
 *
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool System_startWatchdog();

/**
 * Stops watchdog
 *
 * @return        <code>True</code> always.
 * @since         1.0
 */
bool System_stopWatchdog();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
bool System_enableADC();

/**
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */ 
bool System_disableADC();

// Include all headers here so other files only need to include this file.
#include <stdarg.h>
#include <msp430x16x.h>
#include <io.h>
#include <signal.h>
#include <stdlib.h>

#include "ScatterWeb.CC1020.h"
#include "ScatterWeb.Comm.h"
#include "ScatterWeb.Configuration.h"
#include "ScatterWeb.Data.h"
#include "ScatterWeb.Messaging.h"
#include "ScatterWeb.Net.h"
#include "ScatterWeb.String.h"
#include "ScatterWeb.Time.h"
#include "ScatterWeb.Timers.h"

#endif
